//継承してレジスタアドレス機能を拡張する
class I2CUser : public I2C
{
public:
    I2CUser(PinName sda, PinName scl) : I2C(sda, scl){}

    //address       : デバイスアドレス
    //reg           : レジスタアドレス
    //reg_length    :　レジスタアドレス長
    //data          : データ
    //length        : データ長
    int write_with_reg(int address, char* reg, int reg_length, const char* data, int length) {
        int i, status;

        aquire();
        i2c_start(&_i2c);

        LPC_I2C0->MSTDAT = (address & 0xFE);
        LPC_I2C0->MSTCTL |= 0x20;
        if (i2c_wait_SI(&_i2c) == -1)
            return -1;

        status = ((LPC_I2C0->STAT >> 1) & (0x07));
        if (status != 0x02) {
            i2c_stop(&_i2c);
            return I2C_ERROR_NO_SLAVE;
        }

        //レジスタアドレスの送信
        for (i=0; i<reg_length; i++) {
            LPC_I2C0->MSTDAT = reg[i];
            LPC_I2C0->MSTCTL = (1 << 0);
            if (i2c_wait_SI(&_i2c) == -1)
                return -1;

            status = ((LPC_I2C0->STAT >> 1) & (0x07));
            if (status != 0x02) {
                i2c_stop(&_i2c);
                return i;
            }
        }

        //データの送信
        for (i=0; i<length; i++) {
            LPC_I2C0->MSTDAT = data[i];
            LPC_I2C0->MSTCTL = (1 << 0);
            if (i2c_wait_SI(&_i2c) == -1)
                return -1;

            status = ((LPC_I2C0->STAT >> 1) & (0x07));
            if (status != 0x02) {
                i2c_stop(&_i2c);
                return i;
            }
        }

        i2c_stop(&_i2c);

        return length;
    }
private:
    static int i2c_wait_SI(i2c_t *obj) {
        volatile int timeout = 0;
        while (!(LPC_I2C0->STAT & (1 << 0))) {
            timeout++;
            if (timeout > 100000) return -1;
        }
        return 0;
    }
};


class I2CControl
{
public:
    I2CControl(I2C &i2c_reference, char i2c_address): i2c(i2c_reference)
    {
        adr = i2c_address;
    }

    inline int write(unsigned char reg, unsigned char val)
    {
        return write(adr, reg, val);
    }

    unsigned char read(char reg)
    {
        unsigned char recvdat;

        //指定アドレスセット
        i2c.write(adr, &reg, 1);
        //データ読み取り
        i2c.read(adr, buf, 1);

        //データ変換
        recvdat = buf[0];
        return recvdat;
    }

protected:
    I2C &i2c;
    char adr;
    char buf[2];

    int write(char adr, char reg, char data)
    {
        buf[0] = reg;
        buf[1] = data;
        return i2c.write(adr, buf, 2);
    }
};
